@@ -5,12 +5,11 @@
 
 package Test::Future::Deferred;
 
-use v5.10;
 use strict;
 use warnings;
 use base qw( Future );
 
-our $VERSION = '0.48_003';
+our $VERSION = '0.40';
 
 =head1 NAME
 
